#!/bin/sh

CURDIR=`pwd`
LOGFILE="$CURDIR/"`echo $0 | sed -e 's,.*/,,g'`".txt"
date 2>&1 | tee $LOGFILE

ARC=kakasi-2.3.2.tar.gz
SUBARC=Text-Kakasi-1.04.tar.gz

if test -d kakasi ; then
 cd kakasi
fi
BLD=`echo $ARC | sed -e 's/\.tar\.gz$//;s/\.tgz$//; s/\.tar\.bz2$//;s/\.tbz$//; s/\.tar\.Z$//;s/\.tz$//; s/\.gz$//; s/\.bz2$//; s/\.Z$//; s/\.zip$//; s/\.lzh$//; s/\.shar$//;'`
tar xvzf $ARC
cd $BLD

if test -f ../$SUBARC ; then
 tar xvzf ../$SUBARC
 TEXTKAKASI=`echo $SUBARC | sed -e 's/\.tar\.gz$//;s/\.tgz$//; s/\.tar\.bz2$//;s/\.tbz$//; s/\.tar\.Z$//;s/\.tz$//; s/\.gz$//; s/\.bz2$//; s/\.Z$//; s/\.zip$//; s/\.lzh$//; s/\.shar$//;'`
fi

export PREFIX=/kakasi
export TARGET=`gcc -dumpmachine`

export ac_cv_prog_LN_S=ln

if test -e autogen.sh ; then
 ./autogen.sh --prefix=$PREFIX --host=$TARGET 2>&1 | tee -a $LOGFILE
fi
./configure --prefix=$PREFIX --host=$TARGET $* 2>&1 | tee -a $LOGFILE
make 2>&1 | tee -a $LOGFILE
make check 2>&1 | tee -a $LOGFILE
make install 2>&1 | tee -a $LOGFILE

strip $PREFIX/bin/kakasi.exe
strip $PREFIX/bin/atoc_conv.exe
strip $PREFIX/bin/mkkanwa.exe
strip $PREFIX/bin/rdic_conv.exe
strip $PREFIX/bin/wx2_conv.exe

mkdir -p $PREFIX/doc
cp -f doc/* $PREFIX/doc
mkdir -p $PREFIX/man/ja/man1
cp -f doc/kakasi.1 $PREFIX/man/ja/man1

sed -e "s/..//g" -e "s/.//g" doc/kakasi.cat > kakasi.cat.TEMPFILE
nkf -J -s -O kakasi.cat.TEMPFILE $PREFIX/kakasi.sjis
rm -f kakasi.cat.TEMPFILE

cp -f COPYING README README-ja NEWS ONEWS $PREFIX/

if test -z "`type cl 2>&1 | grep 'not found'`" ; then
 rm -f config.cache
 CC="cl -nologo -Gf -W3 -O1 -MD" LD="lib -nologo" ./configure --prefix=$PREFIX --host=i386-windows

 cd lib
 make clean 2>&1 | tee -a $LOGFILE
 make all 2>&1 | tee -a $LOGFILE
## if test -f kakasi.def ; then
##  nmake -f makefile.msc 2>&1 | tee -a $LOGFILE
##  mkdir -p $PREFIX/lib
##  cp -f libkakasi.lib $PREFIX/lib
##  cp -f kakasi.lib kakasi.dll $PREFIX/lib
## else
  OBJS=`ls *.obj`
  lib -nologo -OUT:libkakasi.lib $OBJS
  mkdir -p $PREFIX/lib
  cp -f libkakasi.lib $PREFIX/lib
  echo "LIBRARY \"kakasi\"" > kakasi.def
  echo "EXPORTS" >> kakasi.def
  for F in $OBJS; do
   nm $F | grep 'T _' | sed 's/.* T _/ /' >> kakasi.def
  done
  cl -nologo -Gf -W3 -O1 -MD -LD -Fekakasi.dll $OBJS -link -def:kakasi.def
  mkdir -p $PREFIX/lib
  cp -f kakasi.lib kakasi.dll $PREFIX/lib
  rm -f kakasi.def
## fi
 cd ..

 SYSLIBS=`echo -L$LIB | sed -e "s/\;$//" -e "s/\;/ -L/g"`
 if test -d $TEXTKAKASI ; then
  cd $TEXTKAKASI
  cp -f ../lib/libkakasi.h .
  perl Makefile.PL LIBS="-L../lib -llibkakasi $SYSLIBS" 2>&1 | tee -a $LOGFILE
  nmake 2>&1 | tee -a $LOGFILE
  nmake test 2>&1 | tee -a $LOGFILE
  tar cvzf Text-Kakasi.tar.gz blib/*
  mv -f Text-Kakasi.tar.gz ../
  nmake ppd 2>&1 | tee -a $LOGFILE
  sed -e 's#<ABSTRACT></ABSTRACT>#<ABSTRACT>Kakasi library module for perl</ABSTRACT>#' -e 's#<AUTHOR></AUTHOR>#<AUTHOR>NOKUBI Takatsugu \&lt;knok@daionet.gr.jp\&gt;</AUTHOR>#' -e 's#<CODEBASE HREF="" />#<CODEBASE HREF="x86/Text-Kakasi.tar.gz" />#' Text-Kakasi.ppd > Text-Kakasi.ppd.TEMPFILE
  mv -f Text-Kakasi.ppd.TEMPFILE Text-Kakasi.ppd
  mv -f Text-Kakasi.ppd ../
  cd ..
 fi

 cd lib
 rm -f *.obj
 rm -f *.lib
 rm -f *.dll
 rm -f *.exp
 rm -f .libs/*.lib
 cd ..

 unset OBJS
 unset SYSLIBS
 rm -f config.cache
 make clean 2>&1 | tee -a $LOGFILE
fi
make clean 2>&1 | tee -a $LOGFILE

if test -f Text-Kakasi.tar.gz ; then
 mkdir -p $PREFIX/x86
 mv -f Text-Kakasi.tar.gz $PREFIX/x86
 mv -f Text-Kakasi.ppd $PREFIX
fi

cd $CURDIR

